@@ -1,10 +1,18 @@
 package Dancer2::Session::DBIC;
 
-use Moo;
 use Dancer2::Core::Types;
-use JSON;
+use DBIx::Class;
+use Scalar::Util 'blessed';
+use Module::Runtime 'use_module';
+use Try::Tiny;
 
-our $VERSION = '0.007';
+our %dbic_handles;
+
+use Moo;
+with 'Dancer2::Core::Role::SessionFactory';
+use namespace::clean;
+
+our $VERSION = '0.100';
 
 =head1 NAME
 
@@ -12,14 +20,15 @@ Dancer2::Session::DBIC - DBIx::Class session engine for Dancer2
 
 =head1 VERSION
 
-0.007
+0.100
 
 =head1 DESCRIPTION
 
 This module implements a session engine for Dancer2 by serializing the session,
 and storing it in a database via L<DBIx::Class>.
 
-JSON was chosen as the serialization format, as it is fast, terse, and portable.
+JSON was chosen as the default serialization format, as it is fast, terse,
+and portable.
 
 =head1 SYNOPSIS
 
@@ -36,6 +45,17 @@ Example configuration:
           resultset: "MySession" # DBIx::Class resultset, defaults to Session
           id_column: "my_session_id" # defaults to sessions_id
           data_column: "my_session_data" # defaults to session_data
+          serializer: "YAML"    # defaults to JSON
+
+Or if you are already using L<Dancer2::Plugin::DBIC> and want to use its
+existing configuration for a database section named 'default' with all else
+set to default in this module then you could simply use:
+
+    session: "DBIC"
+    engines:
+      session:
+        DBIC:
+          db_connection_name: default
 
 =head1 SESSION EXPIRATION
 
@@ -45,18 +65,6 @@ This session engine will not automagically remove expired sessions on the server
 
 =cut
 
-use strict;
-
-# use Dancer2;
-use DBIx::Class;
-use Try::Tiny;
-use Scalar::Util qw(blessed);
-use Class::Load qw( try_load_class );
-
-with 'Dancer2::Core::Role::SessionFactory';
-
-my %dbic_handles;
-
 =head1 ATTRIBUTES
 
 =head2 schema_class
@@ -67,6 +75,21 @@ DBIx::Class schema class, e.g. L<Interchange6::Schema>.
 
 has schema_class => (
     is => 'ro',
+    isa => Str,
+);
+
+=head2 db_connection_name
+
+The L<Dancer2::Plugin::DBIC> database connection name.
+
+If this option is provided then L</schema_class>, L</dsn>, L</user> and
+L</password> are all ignored.
+
+=cut
+
+has db_connection_name => (
+    is  => 'ro',
+    isa => Str,
 );
 
 =head2 resultset
@@ -77,6 +100,7 @@ DBIx::Class resultset, defaults to C<Session>.
 
 has resultset => (
     is => 'ro',
+    isa => Str,
     default => 'Session',
 );
 
@@ -84,10 +108,14 @@ has resultset => (
 
 Column for session id, defaults to C<sessions_id>.
 
+If this column is not the primary key of the table, it should have
+a unique constraint added to it.  See L<DBIx::Class::ResultSource/add_unique_constraint>.
+
 =cut
 
 has id_column => (
     is => 'ro',
+    isa => Str,
     default => 'sessions_id',
 );
 
@@ -99,6 +127,7 @@ Column for session data, defaults to C<session_data>.
 
 has data_column => (
     is => 'ro',
+    isa => Str,
     default => 'session_data',
 );
 
@@ -110,6 +139,7 @@ L<DBI> dsn to connect to the database.
 
 has dsn => (
     is => 'ro',
+    isa => Str,
 );
 
 =head2 user
@@ -142,6 +172,91 @@ has schema => (
     is => 'ro',
 );
 
+=head2 serializer
+
+Serializer to use, defaults to JSON.
+
+L<Dancer2::Session::DBIC> provides the following serializer classes:
+
+=over
+
+=item JSON - L<Dancer2::Session::DBIC::Serializer::JSON>
+
+=item Sereal - L<Dancer2::Session::DBIC::Serializer::Sereal>
+
+=item YAML - L<Dancer2::Session::DBIC::Serializer::YAML>
+
+=back
+
+If you do not use the default JSON serializer then you might need to install
+additional modules - see the specific serializer class for details.
+
+You can also use your own serializer class by passing the fully-qualified class
+name as argument to this option, e.g.: MyApp::Session::Serializer
+
+=cut
+
+has serializer => (
+    is      => 'ro',
+    isa     => Str,
+    default => 'JSON',
+);
+
+=head2 serializer_object
+
+Vivified L</serializer> object.
+
+=cut
+
+has serializer_object => (
+    is  => 'lazy',
+    isa => Object,
+);
+
+sub _build_serializer_object {
+    my $self  = shift;
+    my $class = $self->serializer;
+    if ( $class !~ /::/ ) {
+        $class = __PACKAGE__ . "::Serializer::$class";
+    }
+
+    my %args;
+
+    $args{serialize_options} = $self->serialize_options
+      if $self->serialize_options;
+
+    $args{deserialize_options} = $self->deserialize_options
+      if $self->deserialize_options;
+
+    use_module($class)->new(%args);
+}
+
+=head2 serialize_options
+
+Options to be passed to the constructor of the the C<serializer> class
+as a hash reference.
+
+=cut
+
+has serialize_options => (
+    is      => 'ro',
+    isa     => HashRef,
+    default => sub { {} },
+);
+
+=head2 deserialize_options
+
+Options to be passed to the constructor of the the C<deserializer> class
+as a hash reference.
+
+=cut
+
+has deserialize_options => (
+    is      => 'ro',
+    isa     => HashRef,
+    default => sub { {} },
+);
+
 =head1 METHODS
 
 =cut
@@ -159,7 +274,7 @@ sub _flush {
     my $handle = $self->_dbic;
 
     my %session_data = ($handle->{id_column} => $id,
-                        $handle->{data_column} => $self->_serialize($session),
+                        $handle->{data_column} => $self->serializer_object->serialize($session),
                        );
 
     $self->_rset->update_or_create(\%session_data);
@@ -180,7 +295,7 @@ sub _retrieve {
     my ($self, $session_id) = @_;
     my $session_object;
 
-    $session_object = $self->_rset->find($session_id);
+    $session_object = $self->_rset->find({ $self->id_column => $session_id });
 
     # Bail early if we know we have no session data at all
     if (!defined $session_object) {
@@ -188,11 +303,12 @@ sub _retrieve {
         return;
     }
 
-    my $session_data = $session_object->session_data;
+    my $data_column  = $self->data_column;
+    my $session_data = $session_object->$data_column;
 
     # No way to check that it's valid JSON other than trying to deserialize it
     my $session = try {
-        $self->_deserialize($session_data);
+        $self->serializer_object->deserialize($session_data);
     } catch {
         die "Could not deserialize session ID: $session_id - $_";
         return;
@@ -219,7 +335,7 @@ sub _destroy {
         return;
     }
 
-    $self->_rset->find($id)->delete;
+    $self->_rset->find({ $self->id_column => $id})->delete;
 }
 
 # Creates and connects schema
@@ -251,6 +367,15 @@ sub _dbic {
             $handle->{schema} = $schema->();
         }
     }
+    elsif ( $self->db_connection_name ) {
+        try {
+            use_module('DBICx::Sugar');
+        }
+        catch {
+            die "db_connection_name given but DBICx::Sugar fails to load $_";
+        };
+        $handle->{schema} = DBICx::Sugar::schema($self->db_connection_name);
+    }
     elsif (! defined $self->schema_class) {
         die "No schema class defined.";
     }
@@ -290,13 +415,22 @@ sub _load_schema_class {
 
     if ($schema_class) {
         $schema_class =~ s/-/::/g;
-        my ($ok,$err) = try_load_class($schema_class);
-        die "Could not load schema_class $schema_class: $err" unless $ok;
+        try {
+            use_module($schema_class);
+        }
+        catch {
+            die "Could not load schema_class $schema_class: $_";
+        };
         $schema_object = $schema_class->connect(@conn_info);
     } else {
         my $dbic_loader = 'DBIx::Class::Schema::Loader';
-        my ($ok,$err) = try_load_class($dbic_loader);
-        die "You must provide a schema_class option or install $dbic_loader." unless $ok;
+        try {
+            use_module($dbic_loader);
+        }
+        catch {
+            die
+              "You must provide a schema_class option or install $dbic_loader.";
+        };
         $dbic_loader->naming('v7');
         $schema_object = DBIx::Class::Schema::Loader->connect(@conn_info);
     }
@@ -304,38 +438,6 @@ sub _load_schema_class {
     return $schema_object;
 }
 
-# Default Serialize method
-sub _serialize {
-    my $self = shift;
-    my $session = shift;
-
-#    my $settings = setting('session_options');
-
-#    if (defined $settings->{serializer}) {
-#        return $settings->{serializer}->({%$self});
-#    }
-
-    # A session is by definition ephemeral - Store it compactly
-    # This is the Dancer2 function, not from JSON.pm
-    my $json = JSON->new->allow_blessed->convert_blessed;
-    return $json->encode($session);
-}
-
-
-# Default Deserialize method
-sub _deserialize {
-    my ($self, $json) = @_;
-#    my $settings = setting('session_options');
-
-#    if (defined $settings->{deserializer}) {
-#        return $settings->{deserializer}->($json);
-#    }
-
-    # This is the Dancer2 function, not from JSON.pm
-    my $json_obj = JSON->new->allow_blessed->convert_blessed;
-    return $json_obj->decode($json);
-}
-
 =head1 SEE ALSO
 
 L<Dancer2>, L<Dancer2::Session>
@@ -346,8 +448,15 @@ Stefan Hornburg (Racke) <racke@linuxia.de>
 
 =head1 ACKNOWLEDGEMENTS
 
-Based on code from L<Dance::Session::DBI> written by James Aitken
-and code from L<Dance::Plugin::DBIC> written by Naveed Massjouni.
+Based on code from L<Dancer::Session::DBI> written by James Aitken
+and code from L<Dancer::Plugin::DBIC> written by Naveed Massjouni.
+
+Peter Mottram, support for JSON, YAML, Sereal and custom
+serializers, GH #8, #9.
+
+Rory Zweistra, GH #9.
+
+Andy Jack, GH #2.
 
 =head1 COPYRIGHT AND LICENSE
 
